        PROGRAM LTMA
c-----------------------------------------------------------------------
c  Estimates the latent trait mixture model described by Uebersax
c  and Grove (1993).
c-----------------------------------------------------------------------
c  Note:  Uses STEPIT.FOR subroutine
c
c  Author:    John Uebersax
c  Written:   03/25/94
c  Revised:   Change(s)
c  --------   ----------------------------------------------------------
c  03/25/94   Lahey version:  fix PHI1 and ERFCC
c  03/14/96   L77P version
c  04/18/97   Various simplifications
c  04/18/97   Within-class rho terms
c  04/18/97   This version mailed out
c  03/07/99   Various minor changes
c             This is a good version for dichotomous data
c  03/10/99   Allow 3 items and 3 rating levels
c  03/16/99   Disable alternative decision rule statistics (varying
c              panel) until debugged
c-----------------------------------------------------------------------
c  LIST BUGS HERE:  >> problem with 0**0 undefined; must fix for
c                      varying panel models
c-----------------------------------------------------------------------
c  OPTIONS
c
c    IOP1   Use previous start values (1=yes)
c    IOP2   Indexed frequencies (1=yes)
c    IOP3   Ogive form (1=nrml; 2=nrml approx; 3=logistic)
c    IOP4   Equal measurement error (1=yes)
c    IOP5   Print frequencies (1=yes)
c    IOP6   Calculate rater accuracy indices (1=yes)
c    IOP7   Calculate recruitment probabilities (1=yes)
c    IOP8   Fixed (0) or varying (1) panel
c-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        COMMON /FASTER/ JVARY
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS

        OPEN (UNIT=5, FILE='INPUT.LTM' )
        OPEN (UNIT=1, FILE='DATA.LTM'  )
        OPEN (UNIT=2, FILE='OUTPUT.LTM')

        READ  (5, 4999) TITLE
        READ  (5, 5000) IMAX

C    Use previous start values (1=yes)

        READ  (5, 5000) IOP1

C    Indexed frequencies (1=yes)

        READ  (5, 5000) IOP2

        READ  (5, 5000) NRES
        READ  (5, 5000) NR
        READ  (5, 5000) MC
        READ  (5, 5000) NDIS

C    Ogive form (1=normal; 2=normal approx; 3=logistic)

        READ (5, 5000) IOP3

C    Equal measurement error (1=yes)

        READ (5, 5000) IOP4
        NA = NR
        IF (IOP4 .EQ. 1) NA = 1

C    Print frequencies (1=yes)

        READ (5, 5000) IOP5

C    Calculate rater accuracy (1=yes)

        READ (5, 5000) IOP6

C    Calculare recruitment probabilities (1=yes)

        READ (5, 5000) IOP7

C    Fixed (0) or varying (1) panel

        READ (5, 5000) IOP8
        IF (IOP8 .EQ. 1) NA = 1
        NS = NR
        IF (IOP8 .EQ. 1) NS = 1
C
C    Equal measurement error not applicable to varying panel design
C
        IF (IOP8 .EQ. 1) IOP4 = 0
C
C    Error message
C
        IF (MC .GT. 2 .AND. IOP8 .EQ. 1) THEN
          WRITE (6, 6000)
          STOP
        END IF

C    Specify parameters to fix

        NFIXED = 0
        DO 12 I = 1, 10
        READ (5, 5000) NF(I)
        IF (NF(I) .NE. 0) NFIXED = NFIXED + 1
   12   CONTINUE

        NV   = 4 + NA + NS * (MC - 1)
        NEST = NV - NFIXED

C    Input start values

        IF (IOP1 .EQ. 1) GOTO 41

C    Get values from INPUT

        READ (5,5001) DELTAM
        INDX     =    1
        X(INDX)  =    DELTAM

        READ (5,5001) SD(1)
        INDX     =    INDX + 1
        X(INDX)  =    SD(1)

        READ (5,5001) SD(2)
        INDX     =    INDX + 1
        X(INDX)  =    SD(2)

        READ (5,5001) R(2)
        INDX     =    INDX + 1
        X(INDX)  =    R(2)

        DO 25 I  =    1, NA
        READ (5,5001) A(I)
        INDX     =    INDX + 1
        X(INDX)  =    A(I)
   25   CONTINUE

        DO 40 I  =    1, NS
        DO 40 J  =    2, MC
        READ (5,5001) B(I, J)
        INDX     =    INDX + 1
        X(INDX)  =    B(I, J)
   40   CONTINUE
        call startv(1)
        GOTO 45

C    Instead use estimates from last run

   41   CALL STARTV(0)

C    Input observed data

   45   CALL DATAIN
        CALL INIT
c       call report(0)

C    Call STEPIT

        CALL STEPIT

C    Write results

        CALL REPORT (1)

 4999   FORMAT(18A4 )
 5000   FORMAT(I4   )
 5001   FORMAT(D10.2)
 6000   FORMAT(' <<ERROR>>  Varying panel design requires dichotomous',
     .    /,   '            data (MC = 2); run stopped.'              )

        END

        SUBROUTINE OBJFUN
C--8/27/92--------------------------------------------------------------
C  Objective function:
C  Maximize the likelihood ratio chi-square, which has the same effect
C  as maximizing the ln-likelihood.
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        COMMON /FASTER/ JVARY
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS

        PI = 3.141592653589D0

C    Unload transfer vector

C       mean separation

        INDX     =  1
        DELTAM   =  X(INDX)
        RM(1)    =  0.0 - DELTAM
        RM(2)    =  0.0 + DELTAM

C       standard deviations

        INDX     =  INDX + 1
        SD(1)    =  X(INDX)
        INDX     =  INDX + 1
        SD(2)    =  X(INDX)

C       prevalences

        INDX     =  INDX + 1
        R(2)     =  X(INDX)
        R(1)     =  1.0 - R(2)

C       measurement error parameter(s)

        DO  5 I = 1, NA
        INDX     =  INDX + 1
        A(I)     =  X(INDX)
    5   CONTINUE

C       thresholds

        DO 10 I  =  1, NS
        DO 10 J  =  2, MC
        INDX     =  INDX + 1
        B(I, J)  =  X(INDX)
   10   CONTINUE

C    Calculate probability density functions

        CONST1 = 1.0 /(SD(1) * (2 * PI) ** .5)
        CONST2 = 1.0 /(SD(2) * (2 * PI) ** .5)

        SUM1 = 0.
        SUM2 = 0.
        DO 30 I = 1, NRES
        Z1 = (T(I) - RM(1)) / SD(1)
        Z2 = (T(I) - RM(2)) / SD(2)
        F(1, I) = 0.
        F(2, I) = 0.
        IF (ABS(Z1) .LE. 10.) F(1, I) = R(1) * CONST1 * EXP(-.5 * Z1**2)
     .    * TINC
        IF (ABS(Z2) .LE. 10.) F(2, I) = R(2) * CONST2 * EXP(-.5 * Z2**2)
     .    * TINC
        SUM1 = SUM1 + F(1, I)
        SUM2 = SUM2 + F(2, I)
   30   CONTINUE
        DO 40 I = 1, NRES
        F(1, I) = F(1, I) * (R(1) / SUM1)
        F(2, I) = F(2, I) * (R(2) / SUM2)
   40   CONTINUE
C
C Calculate rater ogives
C
        DO 50 I = 1, NS
        AA = A(I)
        IF (IOP4 .EQ. 1) AA = A(1)
        DO 50 J = 1, NRES
        DO 50 K = 2, MC
        SDEV = 1. / AA
        Z = (T(J) - B(I, K)) / SDEV
        IF (IOP3 .EQ. 1) THEN
          P(K, I, J) = PHI1(Z)
            ELSE IF (IOP3 .EQ. 2) THEN
              P(K, I, J) = PHI2(Z)
        ELSE
        P(K, I, J) = 1.0 / (1.0 + EXP(-1.7 * AA * (T(J) - B(I, K))))
        END IF
   50   CONTINUE

        DO 65 I = 1, NS
        DO 65 J = 1, NRES
        P(1, I, J) = 1.0D0 - P(2, I, J)
        IF (MC .EQ. 2) GOTO 65
        DO 60 K = 2, MC - 1
        P(K, I, J) = P(K, I, J) - P(K + 1, I, J)
   60   CONTINUE
   65   CONTINUE
        IF (IOP8 .EQ. 1) GOTO 81

C    Expected frequencies for fixed panel

        DO 80 L = 1, NP
        FE(L) = 0.
        DO 75 I = 1, NRES
        PROD = F(1, I) + F(2, I)
        DO 70 J = 1, NR
        PROD = PROD * P(IVEC(L, J), J, I)
   70   CONTINUE
        FE(L) =  FE(L) + PROD * TOTAL
   75   CONTINUE
   80   CONTINUE
        GOTO 92

C    Expected frequencies for varying panel

   81   DO 90 L  =   1, NP
        LL       =   L - 1
        FE(L)    =   0.
        DO 85 I  =   1, NRES
        PROD     =   F(1, I) + F(2, I)
c       PROD     =   PROD * (P(2, 1, I)**LL) * (P(1, 1, I)**(NR - LL))
c
c to correct lahey fortran bug(?) which leaves 0**0 undefined
c
           if (ll .eq. 0) then
             xxp2 = 1.
           else
             xxp2 = p(2, 1, i)**ll
           end if

           if (nr .eq. ll) then
             xxp1 = 1.
           else
             xxp1 = p(1, 1, i)**(nr - ll)
           end if

           prod     =   prod * xxp2 * xxp1

        FE(L)    =   FE(L) + ICOEF(L) * PROD * TOTAL
   85   CONTINUE
   90   CONTINUE

C    Evaluate fit

   92   CHISQL = 0.0
        DO 120 II = 1, NP
        IF (FO(II) .LT. 0.000001) GOTO 100
        CHISQL = CHISQL + 2 * FO(II) * DLOG(FO(II)/FE(II))
  100   CONTINUE
        CHISQ = CHISQL
  120   CONTINUE

        ICALLS = ICALLS + 1
        WRITE (6,6000) ICALLS, CHISQ
 6000   FORMAT(1X, I5, 5X, F16.8)
        IF (ICALLS .LT. IMAX) GOTO 200
        CALL REPORT (0)
        STOP
  200   CONTINUE
        RETURN
        END

        SUBROUTINE INIT
C--8/27/92--------------------------------------------------------------
C  VARIOUS INITIALIZATION TASKS
C
C  Revised  Changes
C  -------  ------------------------------------------------------------
C 
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS


        ICALLS = 0
        WRITE (6,6000)
C
C    Initialize latent trait levels
C
        TINC   =  20.0 / FLOAT(NRES - 1)
        XXINC  = -10.0 - TINC

        DO 10 I =  1, NRES
        XXINC  =  XXINC + TINC
        T(I)   =  XXINC
   10   CONTINUE

        IF (IOP2 .EQ. 1) GOTO 50
        IF (IOP8 .EQ. 1) GOTO 30

C    Generate fixed-panel response pattern vectors

        DO 20 I = 1, NR
        II = NR + 1 - I
        INDX = 0
        DO 15 J = 1, MC**(I-1)
        DO 15 K = 1, MC
        IF (II .EQ. 1) LL = 1
        IF (II .GT. 1) LL = MC**(II - 1)
        DO 15 L = 1, LL
        INDX = INDX + 1
        IVEC(INDX, I) = K
   15   CONTINUE
   20   CONTINUE
        GOTO 50

C  Generate combination terms

   30   DO 35 L   =  1, NR + 1
        ICOEF(L)  =  0
   35   CONTINUE
        ICOEF(1)  =  1
        DO 40 J   =  1, NR
        DO 40 I   =  1, NR
        II        =  NR + 2 - I
        ICOEF(II) =  ICOEF(II) + ICOEF(II - 1)
   40   CONTINUE

C    See STEPIT documentation

   50   DO 55  I     =  1, NV
        XMAX   ( I)  =  0.0
        XMIN   ( I)  =  0.0
        DELMIN ( I)  =  0.0
        DELTAX ( I)  =  X(I) * (.01)
   55   MASK   ( I)  =  0

C    These variables make STEPIT happy

        NTRACE    =    0
c       MATRIX    =  105
c
c  do not calculate standard errors in stepit
c
        matrix    =  151


C    Fix some parameters

        DO 60 I = 1, 10
        IF (NF(I) .NE. 0) MASK( NF(I)) = 1
   60   CONTINUE

C    Constrain some parameters

C       standard deviations

        XMIN(2)   =     .500
        XMAX(2)   =    5.000
        XMIN(3)   =     .500
        XMAX(3)   =    5.000

C       prevalence for class 2

        XMIN(4)   =     .001
        XMAX(4)   =     .999

C       measurement error parameters

        DO 65  I  =  5, 4 + NA
        XMIN(I)   =     .010
        XMAX(I)   =   10.000
   65   CONTINUE

C       threshold parameters

        DO 70 I   = 5 + NA, NV
        XMIN(I)   =  -10.000
        XMAX(I)   =   10.000
   70   CONTINUE
        RETURN
 6000   FORMAT(1X, 'BEGIN')
        END

        SUBROUTINE STARTV (ICODE)
C-----------------------------------------------------------------------
C SUBROUTINE STARTV -- INPUT/OUTPUT START VALUES
C
C    Parameter    Value    Explanation
C    ---------    -----    --------------------------------
C    ICODE          0      Read start values to START
C                   1      Write start values to STARt
C
C    Revised   Change(s)
C    -------   --------------------------------------------------------
C
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS

C
C    Read starting values from START
C
        OPEN (UNIT=3, FILE='START.LTM')

        IF (ICODE .EQ. 1) GOTO 50

C    Read start values

        DO 10 I = 1, NV
        READ (3, *) X(I)
   10   CONTINUE
        CLOSE (3)
        RETURN

C    Write start values

   50   DO 60 I = 1, NV
        WRITE (3, 3000) X(I)
   60   CONTINUE
        CLOSE (3)
        RETURN
 3000   FORMAT(F20.10)
        END

        DOUBLE PRECISION FUNCTION PHI1(Z)
C--6/17/92--------------------------------------------------------------
C  Use ERFCC to calculate normal cdf value
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        PHI1=ERFCC(-Z/DSQRT(2.))/2.
        RETURN
        END

        DOUBLE PRECISION FUNCTION ERFCC(X)
C-----------------------------------------------------------------------
C  Returns the complementary error function erfc(x) with fractional
C  error less that 1.2E-7
C
C  W. H. Press et al.,  Numerical Recipes (FORTRAN version).
C  Cambridge:  Cambridge University Press, 1989; p. 164
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
C
C Comment -- may need underflow logic
C
        Z=DABS(X)
           T=1./(1.+0.5*Z)
            ERFCC=T*DEXP(-Z*Z-1.26551223+T*(1.00002368+T*(.37409196+
     *        T*(.09678418+T*(-.18628806+T*(.27886807+T*(-1.13520398+
     *        T*(1.48851587+T*(-.82215223+T*.17087277)))))))))
        IF (X.LT.0.) ERFCC=2.-ERFCC
        RETURN
        END

        DOUBLE PRECISION FUNCTION PHI2(Z)
C--7/01/92--------------------------------------------------------------
C  Use Abramowitz & Stegun Eq. 26.2.18 to approximate normal cdf value
C-----------------------------------------------------------------------
          IMPLICIT DOUBLE PRECISION (A-H, O-Z)
          DOUBLE PRECISION C(4)
          DATA C / .196854,
     .             .115194,
     .             .000344,
     .             .019527/
          X    =  ABS(Z)
          PHI2 = .5 / (1. + C(1)*X + C(2)*X**2 + C(3)*X**3 +
     .                      C(4)*X**4)**4
          IF (Z .GE. 0) PHI2 = 1. - PHI2
          RETURN
          END

        SUBROUTINE DATAIN
C-----------------------------------------------------------------------
C SUBROUTINE DATAIN -- INPUT OBSERVED FREQUENCIES
C
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS

        TOTAL  = 0.0
        IF (IOP8 .EQ. 1) GOTO 25
        NCELLS = MC**NR
        IF (IOP2 .EQ. 1) GOTO 10

C    Read non-indexed, full crossclassification table

        READ (1, *) (FO(I), I = 1, NCELLS)
        NP      =  NCELLS
        GOTO 30

C    Read indexed frequency table

   10   READ (1, *)  NP
        DO 20 I = 1, NP
        READ (1, *) (IVEC(I, J), J = 1, NR), FO(I)
   20   CONTINUE
        GOTO 30

C    Read frequencies for varying panel

   25   NCELLS   =   NR + 1
        NP       =   NCELLS
        READ (1, *)  (FO(I), I = 1, NCELLS)

   30   TOTAL   = 0.0
        DO 40 I = 1, NP
        TOTAL   = TOTAL + FO(I)
   40   CONTINUE
        RETURN
        END

        SUBROUTINE REPORT (ICODE)
C--8/27/92--------------------------------------------------------------
C  SUBROUTINE REPORT -- WRITE RESULTS TO OUTPUT
C
C  Revised   Changes
C  -------   -----------------------------------------------------------
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        DIMENSION       TMEAN(9), JVEC(5), PJ(21, 3)
        DIMENSION       RHO2(8)
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        COMMON /BLOCK1/ CHISQL ,    DELTAM ,     TITLE(18),     RM(2)  ,
     .                  SD(2)  ,    R(2)   ,     A(9)     ,     B(8, 5),
     .                  MC     ,    NA     ,     NDIS     ,     NEST   ,
     .                  NP     ,    NR     ,     NRES     ,     NS     ,
     .                  TINC   ,    TOTAL  ,     RHO(8)   ,     NF(10) ,
     .                  IOP1   ,    IOP2   ,     IOP3     ,     IOP4   ,
     .                  IOP5   ,    ICOEF(21),
     .                  IOP6   ,    IOP7     ,   IOP8     ,
     .                  T( 51) ,    F(2,  51),   P( 5,  8,  51),
     .                  FO(256),    FE(256)  ,   IVEC(256, 8), NCELLS

C    Problem information

        WRITE (2, 2100) TITLE
        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2120)
        WRITE (2, 2110)
        WRITE (2, 2125) NR
        WRITE (2, 2130) MC
        WRITE (2, 2135) NRES
        WRITE (2, 2140) NDIS
        WRITE (2, 2145) INT(TOTAL)
        WRITE (2, 2147) NP
        WRITE (2, 2110)
        WRITE (2, 2150)
        WRITE (2, 2110)
        IF (IOP1 .EQ. 1) WRITE (2, 2155)

        IF (IOP8 .EQ. 1) THEN
          WRITE (2, 2160)
        ELSE IF (IOP2 .EQ. 1) THEN
          WRITE (2, 2161)
        ELSE
          WRITE (2, 2162)
        END IF

        IF (IOP3 .EQ. 1) WRITE (2, 2165)
        IF (IOP3 .EQ. 2) WRITE (2, 2170)
        IF (IOP3 .EQ. 3) WRITE (2, 2175)
        IF (IOP4 .EQ. 1) WRITE (2, 2180)
        IF (IOP6 .EQ. 1) WRITE (2, 2185)

C    Error messages

 2155   FORMAT ( '   Use previous start values     ')
 2160   FORMAT ( '   Varying panel data            ')
 2161   FORMAT ( '   Indexed frequency table       ')
 2162   FORMAT ( '   Full data table input         ')
 2165   FORMAT ( '   Normal ogive                  ')
 2170   FORMAT ( '   Normal approximation ogive    ')
 2175   FORMAT ( '   Logistic ogive                ')
 2180   FORMAT ( '   EME restriction               ')
 2185   FORMAT ( '   Rating accuracy indices       ')

C     Fit statistics

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2200)
        WRITE (2, 2110)
        WRITE (2, 2210) CHISQL, CHISQL
        XLOGL    = 0.
        CHISQP   = 0.
        SUME     = 0.

        DO 10 I  = 1, NP
        IF (FO(I) .GT. 0.000001) THEN
          XLOGL    = XLOGL + FO(I) * DLOG(FE(I) / TOTAL)
        END IF
        CHISQP   = CHISQP + ((FO(I) - FE(I))**2) / FE(I)
        SUME     = SUME + FE(I)
   10   CONTINUE

C    Adjust Pearson chi-square for missing cells

        CHISQP   = CHISQP + TOTAL - SUME
        AIC      = -2. * XLOGL + 2. * FLOAT(NEST)
        SCHWARZ  = -2. * XLOGL + DLOG(TOTAL) * FLOAT(NEST)
        NDF      = NCELLS - 1 - NEST
        WRITE (2, 2220) CHISQP, CHISQP
        WRITE (2, 2230) NEST
        WRITE (2, 2240) NDF
        WRITE (2, 2110)

        if (ndf .gt. 0) then
          WRITE (2, 2250) CHISQP/NDF
        end if

        WRITE (2, 2260) XLOGL, XLOGL
        WRITE (2, 2270) AIC, AIC
        WRITE (2, 2280) SCHWARZ, SCHWARZ
        WRITE (2, 2110)
        WRITE (2, 2290) ICALLS

        IF (ICODE .EQ. 1) WRITE (2, 2040)
        IF (ICODE .EQ. 0) WRITE (2, 2041)

C    Parameter estimates

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2300)
        NDIS = 2
        DO 20 I = 1, NDIS
        WRITE (2, 2310) I, R(I), RM(I), SD(I)
   20   CONTINUE
        WRITE (2, 2110)
        WRITE (2, 2320) 2*DELTAM
C
C    Latent trait variance
C
        GM      =  R(1) * RM(1) + R(2) * RM(2)
        VARW    =  R(1) * SD(1)**2 + R(2) * SD(2)**2
        VARB    =  R(1) * (RM(1) - GM)**2 + R(2) * (RM(2) - GM)**2
        VART    =  VARW + VARB

        WRITE (2, 2322) VART

C    Latent correlation(s)

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2330)

        AMEAN   =  0.0
        RMEAN   =  0.0

        DO 30 I   =  1, NA
        VARE      =  1./A(I)**2
        SDE       =  1./A(I)
        RHOSQ     =  VART / (VART + VARE)
        RHO(I)    =  RHOSQ**.5
C
C  NOTE:  assuming sd = 1 for both distributions
C
        RHO2SQ    =  1. / (1. + VARE)
        RHO2(I)   =  RHO2SQ**.5

        IF (IOP4 .EQ. 0) THEN
          AMEAN   =  AMEAN + A(I)   / NR
          RMEAN   =  RMEAN + RHO(I) / NR
          SMEAN   =  SMEAN + SDE    / NR
        END IF
   30   CONTINUE

        IF (NA .GT. 1) THEN
          DO 40 I = 1, NA
          WRITE (2, 2340) I, A(I), 1./A(I), RHO(I), RHO2(I)
   40     CONTINUE
C         WRITE (2, 2342) AMEAN, SMEAN, RMEAN
        ELSE
c         WRITE (2, 2341) A(1), 1./A(1), RHO(1), RHO2(1)
          write (2, 2370) a(1), 1./a(1), rho(1), rho2(1)
 2370   FORMAT(   7X, 'all ', F10.3, f11.3, f11.3, f11.3)

        END IF

C    Thresholds

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2350)
        DO 50 I = 1, NS
        TMEAN(I)= 0.
        DO 50 J = 2, MC

        IF (IOP8 .EQ. 0) THEN
          WRITE (2, 2355) I, J, B(I, J)
        ELSE
          WRITE (2, 2356) J, B(I, J)
        END IF

        IF (MC .GT. 2) TMEAN(I) = TMEAN(I) + B(I, J) / (MC - 1)
   50   CONTINUE

        IF (MC .GT. 2) THEN
          WRITE (2, 2110)
          DO 60 I = 1, NR
          WRITE (2, 2357) I, TMEAN(I)
   60     CONTINUE
        END IF

C    Observed and expected frequencies

        IF (IOP5 .EQ. 1) THEN

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2400)
        SUMO = 0.
        SUME = 0.
        DO 65 I = 1, NP

        IF (IOP8 .EQ. 0) THEN
          WRITE (2, 2410) I, FO(I), FE(I), (IVEC(I, J), J = 1, NR)
        ELSE
          WRITE (2, 2411) I, FO(I), FE(I), I - 1
        END IF

        SUMO = SUMO + FO(I)
        SUME = SUME + FE(I)
   65   CONTINUE
        WRITE (2, 2110)
        WRITE (2, 2025) TOTAL
        WRITE (2, 2030) SUME

        END IF

        IF (IOP6 .EQ. 0) GOTO 120

C    Rater accuracy indices

        DO 70 I = 1, MC
        JVEC(I) = I
   70   CONTINUE
        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2500)

C    Big loop through alternative positive rating definitions

        DO 85 L = 2, MC

        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2505) (JVEC(I), I = L, MC   )
        WRITE (2, 2510) (JVEC(I), I = 1, L - 1)
        WRITE (2, 2515)

        AVGSE   =  0.
        AVGSP   =  0.
        AVGPVP  =  0.
        AVGPVN  =  0.
        AVGPCT  =  0.

C    Loop through items or raters

        DO 80 I   =   1, NS
           PJPOS     =   0.
           PJNEG     =   0.
           PPOS      =   0.
           PNEG      =   0.
           DO 75 J   =   1, NRES
              XPOS      =   0.
              XNEG      =   0.
              DO 72 K   =   1, L - 1
                 XNEG      =   XNEG + P(K, I, J)
   72         CONTINUE
              DO 73 K   =   L, MC
                 XPOS      =   XPOS + P(K, I, J)
   73         CONTINUE
              PJNEG     =   PJNEG + F(1, J) * XNEG
              PJPOS     =   PJPOS + F(2, J) * XPOS
              PNEG      =   PNEG  + (F(1, J) + F(2, J)) * XNEG
              PPOS      =   PPOS  + (F(1, J) + F(2, J)) * XPOS
   75      CONTINUE
           SE        =   PJPOS / R(2)
           SP        =   PJNEG / R(1)
           PVP       =   PJPOS / PPOS
           PVN       =   PJNEG / PNEG
           PCT       =   R(1) * SP + R(2) * SE
           AVGSE   =   AVGSE  +  SE  / NR
           AVGSP   =   AVGSP  +  SP  / NR
           AVGPVP  =   AVGPVP +  PVP / NR
           AVGPVN  =   AVGPVN +  PVN / NR
           AVGPCT  =   AVGPCT +  PCT / NR

           IF (NS .GT. 1) THEN
             WRITE (2, 2520) I, SE, SP, PVP, PVN, PCT
           ELSE
             WRITE (2, 2521)    SE, SP, PVP, PVN, PCT
           END IF

   80   CONTINUE
        IF (NS .EQ. 1) GOTO 85
        WRITE (2, 2110)
        WRITE (2, 2522) AVGSE, AVGSP, AVGPVP, AVGPVN, AVGPCT
   85   CONTINUE
        IF (IOP8 .EQ. 0) GOTO 120

c begin disable option

        if (iop6 .ne. -999) goto 106
C
C    Accuracy of alternative decision rules (varying panel)
C
        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2530) NR

        DO 100 I = 1, NR + 1
        II = I - 1
        SUM1 = 0.
        SUM2 = 0.
        DO 90 J = 1, NRES
        SUM1 = SUM1 + F(1, J) * P(2, 1, J)**II * P(1, 1, J)**(NR - II)
        SUM2 = SUM2 + F(2, J) * P(2, 1, J)**II * P(1, 1, J)**(NR - II)
   90   CONTINUE
        PJ(I, 1) = ICOEF(I) * SUM1
        PJ(I, 2) = ICOEF(I) * SUM2
        PJ(I, 3) = PJ(I, 1) + PJ(I, 2)
c       WRITE (2, 2600) ICOEF(I), PJ(I, 1), PJ(I, 2), PJ(I, 3)
c2600   FORMAT(I12, 3F16.8)
  100   CONTINUE
        SUM1 = 0.
        SUM2 = 0.
        WRITE (2, 2650) 0, 1.0, 0.0, R(2), 0., R(2)
        DO 105 I = 1, NR + 1
C
C if (I - 1) or fewer votes, diagnose case negative
C
        IF (I .EQ. 0) GOTO 101
        SUM1 = SUM1 + PJ(I, 1)
        SUM2 = SUM2 + PJ(I, 2)
  101   QJN  = SUM1
        QJP  = SUM2
        PJN  = R(1) - QJN
        PJP  = R(2) - QJP
        SE   = PJP / R(2)
        SP   = QJN / R(1)

        IF (I .LT. NR + 1) THEN
          PVP  = PJP / (PJP + PJN)
        ELSE
          PVP  = 0.
        END IF

        PVN  = QJN / (QJN + QJP)
        PCT  = R(1) * SP + R(2) * SE
        WRITE (2, 2650) I, SE, SP, PVP, PVN, PCT
 2650   FORMAT(5X, I5, 5X, 4F8.3, F8.3)
  105   CONTINUE

  106   continue

c  end option disable


C
C    Accuracy of unanimous positive votes (varying panel)
C
        WRITE (2, 2110)
        WRITE (2, 2110)
        WRITE (2, 2540)
        DO 115 L = 1, NR
        PJPOS = 0.
        PJNEG = 0.
        PPOS  = 0.
        PNEG  = 0.
        DO 110 I = 1, NRES
        PJPOS  =  PJPOS + F(2, I) * P(2, 1, I)**L
        PJNEG  =  PJNEG + F(1, I) * (1. - P(2, 1, I)**L)
        PPOS   =  PPOS  + (F(1, I) + F(2, I)) * P(2, 1, I)**L
        PNEG   =  PNEG  + (F(1, I) + F(2, I)) * (1. - P(2, 1, I)**L)
  110   CONTINUE
        SE     =  PJPOS/R(2)
        SP     =  PJNEG/R(1)
        PVP    =  PJPOS/PPOS
        PVN    =  PJNEG/PNEG
        PCT    =  R(1) * SP + R(2) * SE
        WRITE (2, 2550) L, SE, SP, PVP, PVN, PCT
  115   CONTINUE
C
C     New section here
C
  120   CONTINUE

C     Write start values

        CALL STARTV(1)

 2100   FORMAT ('LTMA -- Latent Trait Mixture Analysis',/,
     .          'Version 1.0',/,/,/,
     .          'TITLE:  ', 18A4)
 2110   FORMAT ( '')
 2120   FORMAT ( '1. DESIGN INFORMATION' )
 2125   FORMAT ( '   Number of items or raters:        ', I6 )
 2130   FORMAT ( '   Number of response levels:        ', I6 )
 2135   FORMAT ( '   Number of latent trait levels:    ', I6 )
 2140   FORMAT ( '   Number of latent distributions:   ', I6 )
 2145   FORMAT ( '   Number of cases:                  ', I6 )
 2147   FORMAT ( '   Number of response patterns:      ', I6 )
 2150   FORMAT ( '   Options:'                   )

 2200   FORMAT ( '2. FIT STATISTICS'             )
 2210   FORMAT ( '   Likelihood ratio chi-square:      ', F9.2, F20.6 )
 2220   FORMAT ( '   Pearson chi-square:               ', F9.2, F20.6 )
 2230   FORMAT ( '   Number of estimated parameters:   ', I6          )
 2240   FORMAT ( '   Degrees of freedom:               ', I6          )
 2250   FORMAT ( '   Pearson chi-square / df:          ', F9.2        )
 2260   FORMAT ( '   Log-likelihood:                   ', F7.0, F22.6 )
 2270   FORMAT ( '   AIC:                              ', F7.0, F22.6 )
 2280   FORMAT ( '   Schwarz index:                    ', F7.0, F22.6)
 2290   FORMAT ( '   STEPIT iterations:                ', I6         )
 2040   FORMAT ( '   Convergence achieved')
 2041   FORMAT ( '   Convergence not reached')

 2300   FORMAT ( '3. PARAMETER ESTIMATES'/,/,
     .           '   a. Latent distribution parameters   ',/,/,
     .           '      Distri-      Preva-               Standard  ',/,
     .           '      bibution     lence       Mean     deviation ',/,
     .           '      --------    -------    --------   --------- '  )
 2310   FORMAT(1X, I9, 2X, F12.3, F12.3, F11.3                       )
 2320   FORMAT ( '      Difference between means:',F9.3              )
 2322   FORMAT ( '      Latent trait variance:   ',F9.3              )
 2330   FORMAT ( '   b. Measurement error parameter(s) ',/,/,
     .  '                          Measure-   Latent ',/,
     .  '      Item/     Slope     ment       corre- ',/,
     .  '      rater    (alpha)    error      lation ',
     .  '     Rho2',/,
     .  '      -----    -------    -------    ------ ',
     .  '    ------                   '                 )
 2340   FORMAT(1X,  I9,    4(F11.3)  )
 2341   FORMAT(   7X, 'all ', F10.3, 3(F11.3) )
 2342   FORMAT(/, 7X, 'mean', F10.3, 2(F11.3) )
 2350   FORMAT('   c. Threshold parameters     ',/,/,
     .         '      Item/      Rating                ',/,
     .         '      rater      level      Threshold  ',
     .         '    Label',/,
     .         '      -----      ------      --------  ',
     .         '    -------------------------')
 2355   FORMAT(1X, I9,    I11,     F15.3)
 2356   FORMAT(7X, 'all', I10,     F15.3)
 2357   FORMAT(1X, I9, 8X, 'mean', F14.3)
 2400   FORMAT( '4. OBSERVED/EXPECTED FREQUENCIES ',/,/,
     .  '             Observed       Expected     ', /,
     .  '   Cell      frequency      frequency    ',
     .  '        Rating Pattern    ',/,
     .  '   -----     ---------      ---------    ',
     .  '     -------------------- '                      )
 2410   FORMAT(1X, I6, 1X, F16.8, F16.8, 11X, 9I2         )
 2411   FORMAT(1X, I6, 1X, F16.8, F16.8, 11X, I2, ' positive ratings')
 2500   FORMAT ( '5. RATING ACCURACY', /,/,
     ,           '   a.  Accuracy indices')
 2505   FORMAT('   Positive levels:', 4I3)
 2510   FORMAT('   Negative levels:', 4I3)
 2515   FORMAT (74X, '   ', /,
     .   '   Item/ ', 56X, 'percent       '/,
     .   '   rater     Name                  Se      Sp      Pv+  ',
     .   '   Pv-   correct       '/,
     .   '   -----  --------------------   -----   -----   -----  ',
     .   ' -----   -------       '                                  )
 2520   FORMAT(1X,    I5   , 24X , 4F8.3, F9.3)
 2521   FORMAT(1X, '   all', 23X , 4F8.3, F9.3)
 2522   FORMAT(1X, '  mean', 23X , 4F8.3, F9.3)
 2530   FORMAT('   b. Accuracy of decision rules that require various',
     .  ' numbers of positive', /, '      ratings to classify a case ',
     .  'positive, given a panel of size ',I2, /,/,
     .  '      Required                                            ',/,
     .  '      number of                                           ',
     .  '  ROC ',/,
     .  '      positive                                    percent ',
     .  '  curve',/,
     .  '      ratings       Se      Sp     Pv+     Pv-    correct ',
     .  '  area',/,
     .  '      ---------   -----   -----   -----   -----   ------- ',
     .  '  -----'  )
 2540   FORMAT(3X, 'b. Accuracy of unanimous positive ratings given ',
     .  'panels of various sizes', /,/,
     .  3X,'                                                         ',
     .  'ROC  ',/,
     .  3X,'   Panel                                       percent   ',
     .  'curve',/,
     .  3X,'   size          Se      Sp     Pv+     Pv-    correct   ',
     .  'area ',/,
     .  3X,'   -----       -----   -----   -----   -----   -------   ',
     .  '-----' )
 2550   FORMAT(5X, I5, 5X, 4F8.3, F9.3)
 2025   FORMAT('   Total observed frequency        ', F12.3 )
 2030   FORMAT('   Total expected frequency        ', F12.3 )
        RETURN
        END


        SUBROUTINE STEPIT
C=======================================================================
C  SUBROUTINE STEPIT -- GENERALIZED MAXIMIZATION/MINIMIATION ROUTINE   
C
C  Author:          J. P. Chandler
C                   Physics Department, Indiana University
C  Version:         5.2
C, Written:         August, 1967
C
C  Available from Quantum Chemistry Program Exchange, I. U. Chemistry
C  Department, Bloomington, Indiana
C
C  (Modififed for FORTRAN G OS/360 MVT)
C=======================================================================
c  Begin comments added by John Uebersax:
c
c  John P. Chandler has given permission for noncommercial use of
c  the STEPIT subroutine.  However, if you incorporate this
c  this subroutine into your programs, an appropriate citation
c  is requested.  The relevant citation is:
c
c     Chandler, J. P. (1969).  STEPIT--Finds local minima of a smooth
c     function of several parameters. Computer program abstract.
c     Behavioral Science, 14, 81-82.
c
c  Comments made by me are marked with a lowercase "c".  Those
c  with an uppercase "C" are from the original program.
c-----------------------------------------------------------------------
c  Revisions by John Uebersax
c  Date         Change(s)
c  --------     --------------------------------------------------------
c  10/10/92     Modification for Lahey f77L FORTRAN
c  03/14/96     Modified for Lahey f77P FORTRAN
c  03/26/96     FORMAT changes
c  04/22/96     write matrix of second partial derivatives
c  04/25/96     call subroutine to calculate first derivatives
c               larger DETERMINANT limit
c  07/13/96     clean up standard error stuff
c
c-----------------------------------------------------------------------
c  Glossary:
c
c    IOP1      Suppress (0) or calculate (1) standard errors
c-----------------------------------------------------------------------
c
c  End comments by John Uebersax
C=======================================================================
c     IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)                

      COMMON/STPIT/CHISQ,X(25),XMAX(25),XMIN(25),DELTAX(25),ERR(25,25),D
     1ELMIN(25),MASK(25),NV,NTRACE,MATRIX

      COMMON /FASTER/ JVARY
      COMMON /ICALL/  ICALLS,IMAX

      DIMENSION VEC(70),TRIAL(70),XSAVE(70),CHI(70),DX(70),SECOND(2,2)
      DIMENSION OLDVEC(70),SALVO(70),XOSC(70,15),CHIOSC(15),NFLAT(70)

      open (unit=4, file="step.out")                      
      open (unit=7, file="second.mat")                    

c
c     NVMAX  = 50
c     KW     = 61
      MOSQUE = 15
      RATIO  = 10.0
      COLIN  = 0.99
      NCOMP  = 5
      ACK    = 2.0
c     HUGE   = 1.E65
      SIGNIF = 2.E6                                       
c     SIGNIF = 2.E8                                       
      HUGE   = 1.D65                                      
c
      JVARY=0
   40 IF(NV)290,290,50
   50 NACTIV=0
      DO 150 I=1,NV
      IF(MASK(I))150,60,150
   60 IF(SIGNIF*DABS(DELTAX(I))-DABS(X(I))) 70,70,100
   70 IF(X(I))90,80,90
   80 DELTAX(I)=0.01
      GO TO 100
   90 DELTAX(I)=0.01*X(I)
  100 IF(DELMIN(I))120,110,120
  110 DELMIN(I)=DELTAX(I)/SIGNIF
  120 IF(XMAX(I)-XMIN(I))130,130,140
  130 XMAX(I)=HUGE
      XMIN(I)=-HUGE
  140 NACTIV=NACTIV+1
      VACD=DMIN1(XMAX(I),X(I))
      X(I)=DMAX1(XMIN(I),VACD)
  150 CONTINUE
      COMPAR=0.0
      IF(NACTIV-1)160,190,180
  160 DO 170 J=1,NV
  170 MASK(J)=0
      GO TO 50
  180 A=NACTIV
      SUB=2.0/(A-1.0)
      P=2.0*(1.0/DSQRT(A)/(1.0-0.5**SUB)-1.0)
      COMPAR=DMIN1(9.99D-1,DABS((1.0-(1.0-COLIN)**SUB)*(1.0+P*(1.-COLIN)
     1)))
  190 IF(NTRACE)280,200,200
200   WRITE (4,210)
  210 FORMAT(1X,' ')
c 210 FORMAT(1X,'ENTER SUBROUTINE STEPIT.  COPYRIGHT 1965 J. P. CHANDLER
c    1, INDIANA UNIVERSITY',//,1X,'INITIAL VALUES....'/)
c      WRITE(6,220)(MASK(J),J=1,NV)
c  220 FORMAT(/(I6,6X)/(4X,10I12))
c      WRITE(6,230)(X(J),J=1,NV)
c  230 FORMAT(/F10.4/(10X,10F10.4))
c      WRITE(6,240)(XMAX(J),J=1,NV)
c  240 FORMAT(/F10.4/(10X,10F10.4))
c      WRITE(6,250)(XMIN(J),J=1,NV)
c  250 FORMAT(/F10.4/(10X,10F10.4))
c      WRITE(6,260)(DELTAX(J),J=1,NV)
c  260 FORMAT(/E12.4/(10X,10F10.4))
c      WRITE(6,270)(DELMIN(J),J=1,NV)
c  270 FORMAT(/E15.4/(10X,10F10.4))
  280 CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=1
      JOCK=1
  290 IF(NTRACE)320,300,300
c
c XXXMOD JULY 1989
c
  300 I = I
c
c 300 WRITE(6,310)NV,NACTIV,MATRIX,NCOMP,RATIO,ACK,COLIN,COMPAR,CHISQ
c 310 FORMAT(/1X,I3,I3,I4,
c    1I2/1X,F5.1,10X,F5.1,F6.3,10X,
c    2F6.3/1X,F10.8)
  320 IF(NV)2150,2150,330
  330 IF(NTRACE)360,360,340
c
c XXXMOD JULY 1989
c
  340 I = I
c
c 340 WRITE(6,350)
c 350 FORMAT(/10(1X,1H*))
c
  360 DO 370 I=1,NV
      DX(I)=DELTAX(I)
      VEC(I)=0.
      DO 370 J = 1,20
  370 ERR(I,J)=0.
      CHIOLD=CHISQ
      NOSC=0
C
  380 NCIRC=0
      NZIP=0
C
C  MAIN DO LOOP FOR CYCLING THROUGH THE VARIABLES.
C  FIRST TRIAL STEP WITH EACH VARIABLE IS SEPARATE.
  390 NACK=0
      DO 1350 I=1,NV
      OLDVEC(I)=VEC(I)
      VEC(I)=0.0
      TRIAL(I)=0.0
      IF(MASK(I))400,410,400
  400 VEC(I)=-0.0
      NFLAT(I)=1
      GO TO 1350
  410 NACK=NACK+1
      SAVE=X(I)
      IF(SIGNIF*DABS(DX(I))-DABS(X(I))) 580,580,420
  420 X(I)=SAVE+DX(I)
      JVARY=0
      IF(JOCK)440,440,430
  430 JOCK=0
      JVARY=I
  440 NFLAG=1
      IF(X(I)-XMIN(I))460,460,450
  450 IF(X(I)-XMAX(I))470,460,460
  460 NFLAG=NFLAG+3
      GO TO 490
  470 CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      IF(ICALLS.LE.IMAX)GO TO 10
      RETURN
   10 CONTINUE
      NF=NF+1
      JVARY=I
      CHIME=CHISQ
      IF(CHISQ-CHIOLD)620,480,490
  480 NFLAG=NFLAG+1
  490 X(I)=SAVE-DX(I)
      IF(X(I)-XMIN(I))590,590,500
  500 IF(X(I)-XMAX(I))510,590,590
  510 CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      JVARY=I
      IF(CHISQ-CHIOLD)610,520,530
  520 NFLAG=NFLAG+1
  530 IF(NFLAG-3)540,580,590
  540 IF((CHISQ-CHIME)*(CHIME-2.*CHIOLD+CHISQ))550,590,550
  550 TRIAL(I)=.5*DX(I)*(CHISQ-CHIME)/(CHIME-2.*CHIOLD+CHISQ)
      VEC(I)=TRIAL(I)/DABS(DX(I))
      NFLAT(I)=0
      X(I)=SAVE+TRIAL(I)
      CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      IF(CHISQ-CHIOLD)560,570,570
  560 CHIOLD=CHISQ
      JOCK=1
      GO TO 600
  570 TRIAL(I)=0.0
      VEC(I)=0.0
      GO TO 590
  580 VEC(I)=-0.0
      NFLAT(I)=1
  590 X(I)=SAVE
  600 NCIRC=NCIRC+1
      IF(NCIRC-NACTIV)690,1430,1430
  610 DX(I)=-DX(I)
C
C  A LOWER VALUE HAS BEEN FOUND.  HENCE THIS VARIABLE WILL CHANGE.
C
  620 NCIRC=0
      DEL=DX(I)
  630 CHIME=CHIOLD
      CHIOLD=CHISQ
      VEC(I)=VEC(I)+DEL/DABS(DX(I))
      NFLAT(I)=0
      TRIAL(I)=TRIAL(I)+DEL
      DEL=ACK*DEL
      SAVE=X(I)
      X(I)=SAVE+DEL
      IF(X(I)-XMIN(I))680,680,640
  640 IF(X(I)-XMAX(I))650,680,680
  650 CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      IF(CHISQ-CHIOLD)630,660,660
660    ZZZ=ACK*CHIME-(ACK+1.0)*CHIOLD+CHISQ
      IF(ZZZ .EQ. 0.) GO TO 661
      CINDER=(0.5/ACK)*(ACK**2*CHIME-(ACK**2-1.0)*CHIOLD-CHISQ)/ZZZ
      GO TO 662
661   CINDER =0.0
662   X(I)=SAVE+CINDER*DEL
      CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      IF(CHISQ-CHIOLD)670,680,680
  670 CHIOLD=CHISQ
      TRIAL(I)=TRIAL(I)+CINDER*DEL
      VEC(I)=VEC(I)+CINDER*DEL/DABS(DX(I))
      GO TO 690
  680 X(I)=SAVE
  690 IF(NZIP-1)1340,700,700
  700 IF(DABS(VEC(I))-ACK) 750,710,710
  710 DX(I)=ACK*DABS(DX(I))
      VEC(I)=VEC(I)/ACK
      OLDVEC(I)=OLDVEC(I)/ACK
      DO 720 J=1,MOSQUE
  720 ERR(I,J)=ERR(I,J)/ACK
      IF(NTRACE)750,750,730
  730 WRITE(4,740)I,DX(I)
  740 FORMAT(1X,I3,F10.5)
  750 SUMO=0.0
      SUMV=0.0
      DO 760 J=1,NV
      SUMO=SUMO+OLDVEC(J)**2
  760 SUMV=SUMV+VEC(J)**2
      IF(SUMO*SUMV)1340,1340,770
  770 SUMO=DSQRT(SUMO)
      SUMV=DSQRT(SUMV)
      COSINE=0.0
      DO 780 J=1,NV
  780 COSINE=COSINE+(OLDVEC(J)/SUMO)*(VEC(J)/SUMV)
      IF(NZIP-1)1340,790,800
  790 IF(NACK-NACTIV)1340,820,820
  800 IF(NACK-NACTIV)820,810,810
  810 IF(NZIP-NCOMP)820,830,830
  820 IF(COSINE-COMPAR)1340,830,830
C
C  SIMON SAYS, TAKE AS MANY GIANT STEPS AS POSSIBLE...
C
  830 IF(NTRACE)860,860,840
  840 WRITE(4,850)CHIOLD,(VEC(J),J=1,I)
  850 FORMAT(1X,F10.8,5X,10F9.2/,1X,10F9.2)
  860 NGIANT=0
      NTRY=0
      NRETRY=0
      KL=1
      NOSC=NOSC+1
      IF(NOSC-MOSQUE)890,890,870
  870 NOSC=MOSQUE
      DO 880 K=2,MOSQUE
      CHIOSC(K-1)=CHIOSC(K)
      DO 880 J=1,NV
      XOSC(J,K-1)=XOSC(J,K)
  880 ERR(J,K-1)=ERR(J,K)
  890 DO 900 J=1,NV
      XOSC(J,NOSC)=X(J)
  900 ERR(J,NOSC)=VEC(J)/SUMV
      CHIOSC(NOSC)=CHIOLD
      IF(NOSC-3)960,910,910
C
C  SEARCH FOR A PREVIOUS SUCCESSFUL GIANT STEP IN A DIRECTION MORE
C  NEARLY PARALLEL TO THE DIRECTION OF THE PROPOSED STEP THAN WAS THE
C  IMMEDIATELY PREVIOUS ONE.
C
  910 COXCOM=0.0
      DO 920 J=1,NV
  920 COXCOM=COXCOM+ERR(J,NOSC)*ERR(J,NOSC-1)
      NAH=NOSC-2
  930 NTRY=0
      DO 950 K=KL,NAH
      NRETRY=NAH-K
      COSINE=0.0
      DO 940 J=1,NV
  940 COSINE=COSINE+ERR(J,NOSC)*ERR(J,K)
      IF(COSINE-COXCOM)950,950,970
  950 CONTINUE
  960 CHIBAK=CHI(I)
      GO TO 1020
  970 NTRY=1
      KL=K+1
      IF(NTRACE)1000,1000,980
  980 NT=NOSC-K
      WRITE(4,990)NT
  990 FORMAT(/1X,I2)
 1000 DO 1010 J=1,NV
      SALVO(J)=TRIAL(J)
 1010 TRIAL(J)=(X(J)-XOSC(J,K))/ACK
      CHIBAK=CHIOLD+(CHIOSC(K)-CHIOLD)/ACK
C
 1020 DO 1040 J=1,NV
      XSAVE(J)=X(J)
      TRIAL(J)=ACK*TRIAL(J)
      IF(MASK(J))1040,1030,1040
 1030 TRAAL=X(J)+TRIAL(J)
      TRAIL=DMIN1(TRAAL,XMAX(J))
      X(J)=DMAX1(TRAIL,XMIN(J))
 1040 CONTINUE
      JOCK=0
      JVARY=0
      CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      IF(CHISQ-CHIOLD)1050,1080,1080
 1050 CHIBAK=CHIOLD
      CHIOLD=CHISQ
      NGIANT=NGIANT+1
      IF(NTRACE)1020,1020,1060
 1060 WRITE(4,1070)CHISQ,(X(J),J=1,NV)
 1070 FORMAT(1X,F10.8,/(10(1X,F10.5)))
      GO TO 1020
C
 1080 IF(NRETRY)1100,1100,1090
 1090 IF(NGIANT)1150,1150,1100
 1100 CINDER=(0.5/ACK)*(ACK**2*CHIBAK-(ACK**2-1.0)*CHIOLD-CHISQ)/
     1   (ACK*CHIBAK-(ACK+1.0)*CHIOLD+CHISQ)
      DO 1120 J=1,NV
      IF(MASK(J))1120,1110,1120
 1110 CANDER=XSAVE(J)+CINDER*TRIAL(J)
      XIJ=DMIN1(CANDER,XMAX(J))
      X(J)=DMAX1(XIJ,XMIN(J))
 1120 CONTINUE
      JOCK=0
      JVARY=0
      CALL OBJFUN
      if (icalls .ge. IMAX) return                        
      NF=NF+1
      IF(CHISQ-CHIOLD)1280,1130,1130
 1130 IF(NGIANT)1170,1140,1170
 1140 IF(NTRY)1150,1170,1150
 1150 DO 1160 J=1,NV
      TRIAL(J)=SALVO(J)
 1160 X(J)=XSAVE(J)
      GO TO 1190
 1170 DO 1180 J=1,NV
      TRIAL(J)=TRIAL(J)/ACK
 1180 X(J)=XSAVE(J)
 1190 IF(NTRACE)1240,1240,1200
 1200 WRITE(4,1210)CHIOLD,NGIANT
 1210 FORMAT(1X,F10.8)
      WRITE(4,1220)(X(J),J=1,NV)
 1220 FORMAT((10(1X,F10.5)))
      WRITE(4,1230)
 1230 FORMAT(/1X,' ')
 1240 IF(NGIANT)1250,1250,1310
 1250 IF(NRETRY)1260,1260,930
 1260 IF(NTRY)1270,1330,1270
 1270 NTRY=0
      GO TO 960
C
 1280 CHIOLD=CHISQ
      JOCK=1
      IF(NTRACE)1310,1310,1290
 1290 STEPS=DFLOAT(NGIANT)+CINDER
      WRITE(4,1300)CHIOLD,STEPS
 1300 FORMAT(1X,F10.8)
      WRITE(4,1220)(X(J),J=1,NV)
      WRITE(4,1230)
1310   IF (NTRY) 1320,380,1320
 1320 NOSC=0
      GO TO 380
1330  NOSC=MAX0(NOSC-1,0)
 1340 CHI(I)=CHIOLD
 1350 CONTINUE
C
C  ANOTHER CYCLE THROUGH THE VARIABLES HAS BEEN COMPLETED.
C  PRINT ANOTHER LINE OF TRACES.
C
      IF(NTRACE)1370,1370,1360
 1360 WRITE(4,850)CHIOLD,(VEC(J),J=1,NV)
1370    CONTINUE
 1380 IF(NZIP)1420,1390,1420
 1390 IF(NTRACE)1420,1420,1400
 1400 WRITE(4,1220)(X(J),J=1,NV)
      WRITE(4,1410)
 1410 FORMAT(1H )
 1420 NZIP=NZIP+1
      GO TO 390
C
C  A MINIMUM HAS BEEN FOUND.  PRINT THE REMAINING TRACES.
C
 1430 IF(NTRACE)1450,1450,1440
 1440 WRITE(4,850)CHIOLD,(VEC(J),J=1,I)
 1450 IF(NTRACE)1470,1470,1460
 1460 WRITE(4,1220)(X(J),J=1,NV)
      WRITE(4,1230)
C
C  DECREASE THE SIZE OF THE STEPS FOR ALL VARIABLES.
C
1470   CONTINUE
 1480 NOSC=0
      NGATE=1
      DO 1520 J=1,NV
      IF(MASK(J))1520,1490,1520
 1490 IF(NFLAT(J))1500,1500,1520
 1500 IF(DABS(DX(J))-DABS(DELMIN(J))) 1520,1520,1510
 1510 NGATE=0
 1520 DX(J)=DX(J)/RATIO
      IF(NGATE)1530,1530,1600
 1530 IF(NTRACE)1570,1570,1540
 1540 WRITE(4,1550)(DX(J),J=1,NV)
 1550 FORMAT(10(1X,1H*)/,(10(1X,F10.5)))
      WRITE(4,1560)
 1560 FORMAT(//)
 1570   GO TO 380
c1580 WRITE(6,1590)(DX(J),J=1,NV)
c1590 FORMAT (1X,'OPERAT. TERM.'//10(1X,F10.5))
c
 1600 CHISQ=CHIOLD
      IF(NTRACE)1630,1610,1610
 1610 WRITE(4,1620)NF                                     
      WRITE (4, 1620) NF
C1620 FORMAT(/1X,I5,23H FUNCTION COMPUTATIONS     )
 1620 FORMAT(1X, I5, " FUNCTION COMPUTATIONS")            
      WRITE (*, *) " Convergence achieved"                

C                                                         
C Write current parameter estimates and CHISQ             
C                                                         
      WRITE (4, 2210) (X(J), J = 1, NV)                   
      WRITE (4, 2220) CHISQ                               

1630   CONTINUE
1640  IF (IABS(MATRIX-100)-50) 1650,1650,2160            
1650  IF(NACTIV-NV)2160,1660,2160                        
C
C  COMPUTE THE STANDARD ERRORS AND THE CORRELATIONS.
C
 1660 FAC=RATIO**(MATRIX-100)
      WRITE (*, *) " Calculating standard errors"         
c
c  Reset ICALLS so that program doesn't terminate
c
      ICALLS = 0
      DO 1680 I=1,NV
      DX(I)=DABS(FAC*DX(I))
      XSAVE(I)=X(I)
      JVARY=0
      DO 1670 J=1,2
      X(I)=XSAVE(I)+DX(I)
      CALL OBJFUN
      NF=NF+1
      JVARY=I
      SECOND(1,J)=CHISQ
 1670 DX(I)=-DX(I)
      X(I)=XSAVE(I)
 1680 ERR(I,I)=(SECOND(1,1)-2.0*CHIOLD+SECOND(1,2))/DX(I)**2
      DO 1710 I=2,NV
      IM=I-1
      DO 1710 J=1,IM
      DO 1700 K=1,2
      X(I)=XSAVE(I)+DX(I)
      JVARY=0
      DO 1690 L=1,2
      X(J)=XSAVE(J)+DX(J)
      CALL OBJFUN
      NF=NF+1
      JVARY=J
      SECOND(K,L)=CHISQ
      X(J)=XSAVE(J)
 1690 DX(J)=-DX(J)
      X(I)=XSAVE(I)
 1700 DX(I)=-DX(I)
      ERR(I,J)=0.25*(SECOND(1,1)-SECOND(1,2)-SECOND(2,1)+SECOND(2,2))
     1/DABS(DX(I)*DX(J))
 1710 ERR(J,I)=ERR(I,J)
      IF(NTRACE)1770,1720,1720
 1720 WRITE(4,1730)
C 1730 FORMAT(40HSIZES OF INCREMENTS TO BE USED BELOW....)
 1730 FORMAT(/, "Increment sizes:")                       
      WRITE(4,1740)(DX(J),J=1,NV)
 1740 FORMAT(/(10(1X,F10.5)))
      WRITE(4,1750)
C1750 FORMAT(/45H MATRIX OF THE SECOND PARTIAL DERIVATIVES.... /)
 1750 FORMAT(/, "Second partial derivatives:")            

      do 1751 i = 1, nv                                   
      do 1751 j = 1, i                                    
      write (7, *) err(i, j)                              
 1751 continue                                            

c                                                         
c calculate first derivatives                             
c                                                         
      call first                                          

      DO 1760 I=1,NV
 1760 WRITE(4,1740)(ERR(I,J),J=1,I)
 1770 DO 1780 I=1,NV
      DO 1780 J=1,I
      IF(ERR(I,J))1780,1790,1780
 1780 CONTINUE
      GO TO 1810
 1790 WRITE(4,1800)
 1800 FORMAT(////123H THE ABOVE MATRIX CONTAINS ONE OR MORE ZEROES.  A L
     1ARGER VALUE OF 'MATRIX' SHOULD BE TRIED, TO SEE IF THEY ARE LEGITI
     2MATE.       )
C
C  INVERT THE MATRIX USING SYMINV2 (COMM. OF THE A.C.M. 6, P. 67).
C
 1810 DET=1.0
      DETLOG=0.
      DO 1820 J=1,NV
 1820 SALVO(J)=1.0
      DO 1970 I=1,NV
      BIGAJJ=0.0
      DO 1850 J=1,NV
      IF(SALVO(J))1830,1850,1830
 1830 IF(DABS(ERR(J,J))-BIGAJJ) 1850,1850,1840
 1840 BIGAJJ=DABS(ERR(J,J))
      K=J
 1850 CONTINUE
      IF(BIGAJJ)1870,1860,1870
 1860 DET=0.0
      GO TO 1980
 1870 SALVO(K)=0.0
c      IF (DET .GT. 1.E48 .OR. DET .LT.1.E-48) GO TO 1871
c      IF (DET .GT. 1.D48 .OR. DET .LT.1.D-48) GO TO 1871     

       IF (DET .GT. 1.D64 .OR. DET .LT.1.D-64) GO TO 1871     

      GO TO 1875
1871   WRITE (4,1872)
1872   FORMAT (///,'DETERMINANT COMPUTATION WILL CAUSE OVERFLOW.')
      GO TO 2160
1875  DET=DET*ERR(K,K)
      DETLOG=DETLOG+DLOG(DABS(ERR(K,K)))/2.303
      TRIAL(K)=1.0/ERR(K,K)
      ERR(K,K)=0.0
      XSAVE(K)=1.0
      M=K-1
      IF(M)1910,1910,1880
 1880 DO 1900 J=1,M
      XSAVE(J)=ERR(K,J)
      TRIAL(J)=ERR(K,J)*TRIAL(K)
      IF(SALVO(J))1860,1900,1890
 1890 TRIAL(J)=-TRIAL(J)
 1900 ERR(K,J)=0.0
 1910 M=K+1
      IF(M-NV)1920,1920,1960
 1920 DO 1950 J=M,NV
      XSAVE(J)=ERR(J,K)
      IF(SALVO(J))1860,1930,1940
 1930 XSAVE(J)=-XSAVE(J)
 1940 TRIAL(J)=-ERR(J,K)*TRIAL(K)
 1950 ERR(J,K)=0.0
 1960 DO 1970 J=1,NV
      DO 1970 K=J,NV
 1970 ERR(K,J)=ERR(K,J)+XSAVE(J)*TRIAL(K)
      IF(DET)2000,1980,2020
 1980 WRITE(4,1990)
 1990 FORMAT(////67H ERROR MATRIX IS SINGULAR.  'MATRIX' SHOULD PROBABLY
     1 BE INCREASED.              /////)
      GO TO 2150
 2000 WRITE(4,2010)
 2010 FORMAT(////1X,'ERROR MATRIX IS NEGATIVE DEFINITE.  MATRIX SHOULD
     1 PROBABLY BE DECREASED.')
 2020 IF(NTRACE)2050,2030,2030

c 2030 WRITE(6,2040)DET,DETLOG
c 2040 FORMAT(/1X,F10.5,10X,F10.5)
 2030 WRITE (4, 2040)                                     
 2040 FORMAT(/, "Determinant:")                           
      write (4, *) det                                    

 2050 DO 2090 I=1,NV
      DO 2060 J=1,I
      ERR(I,J)=ERR(I,J)*2.0
 2060 ERR(J,I)=ERR(I,J)
      IF(ERR(I,I))2070,2070,2090
 2070 WRITE(4,2080)ERR(I,I)
 2080 FORMAT(///50H NEGATIVE OR ZERO MEAN SQUARE ERROR ENCOUNTERED...,
     13X,F10.8/39H 'MATRIX' SHOULD PROBABLY BE DECREASED. ///)
 2090 XSAVE(I)=DSIGN(DSQRT(DABS(ERR(I,I))),ERR(I,I))
      IF(NTRACE)2160,2100,2100
 2100 WRITE(4,2110)
c 2110 FORMAT(/////20H STANDARD ERRORS.... )
c2110 FORMAT(/, "Standard Errors:")
c     WRITE(4,1740)(XSAVE(J),J=1,NV)

 2110 FORMAT(/, "Standard Errors:", /, "")                
      do 3000 j = 1, nv                                   
        write (4, 4010) j, x(j), xsave(j), xsave(j)       
 3000 continue                                            
 4010 format(1x, i6, f10.3, f10.4, f10.3)                 
                                                          
C      WRITE(6,2120)                                      
       IF (NV .GT. 1) WRITE (4, 2120)                     
c 2120 FORMAT(/////45H LOWER TRIANGLE OF THE CORRELATION MATRIX.... /)
 2120 FORMAT(/, "Correlation matrix:")                    
      DO 2140 I=2,NV
      IM=I-1
      DO 2130 J=1,IM
 2130 TRIAL(J)=ERR(I,J)/DABS(XSAVE(I)*XSAVE(J))
 2140 WRITE(4,1740)(TRIAL(J),J=1,IM)
 2150 WRITE(4,1620)NF
C
2160  CONTINUE
 2190 JVARY=0
      CALL OBJFUN
      IF(NTRACE)2230,2200,2200
 2200 WRITE(4,2210)(X(J),J=1,NV)
 2210 FORMAT(/1X,/(7(1X,E16.9)))
      WRITE(4,2220)CHISQ
 2220 FORMAT(/, "CHISQ =  ", F10.8)
 2230 CONTINUE
      RETURN
      END

        SUBROUTINE FIRST
c----------------------------------------------------------------------
c  This subroutine added by John Uebersax
c
c  Calculate first derivatives for a user-supplied function using
c  forward finite differences
c----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)

        COMMON /STPIT/  CHISQ, X(25), XMAX(25), XMIN(25), DELTAX(25),
     .                  ERR(25,25), DELMIN(25), MASK(25), NV, NTRACE,
     .                  MATRIX

        XXINC = .00000001

        WRITE (4, 4000)
 4000   FORMAT(/, "First derivatives:", /, "")

        CALL OBJFUN
        F1 = CHISQ

        DO 10 I = 1, NV
           X(I) = X(I) + XXINC
           CALL OBJFUN
           F2 = CHISQ
           X(I) = X(I) - XXINC

           WRITE (4, 4010) I, X(I), (F2 - F1) / XXINC
 4010      FORMAT(I6, F10.3, F16.8)
  10    CONTINUE

        RETURN
        END
